function answer = demo3D11( param )
% demo3D11
%
% Trains a generative model of the cell framework using the four patterns in the 3D HeLa
% dataset from the Murphy Lab

% Created: Ivan E. Cao-Berg
%
% Copyright (C) 2012-2105 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% May 15, 2013 I. Cao-Berg Updated method to support wildcards
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

curr_path = which( mfilename );
curr_path = curr_path(1:end-10);
cd(curr_path);

if nargin == 0
    param = [];
    model = [];
else
    try
        model = param.model;
    catch
        model = [];
    end
end

% framework model was trained at full resolution
model = ml_initparam( model, ...
    struct( 'downsampling', [1,1,1] ) );

% changing the seed will produce a different set of synthetic images
param = ml_initparam( param, ...
    struct( 'seed', 3 ) );

param = ml_initparam( param, ...
    struct( 'preprocess', true ) );

% optional parameter if set to true, will use the processed images found in
% the downloaded tarball
param = ml_initparam( param, ...
    struct( 'faster', false ) );

% optional parameter that if set to true, will traing a framework model
param = ml_initparam( param, ...
    struct( 'train_model', true ) );

% optional parameter that if set to true, will synthesize images from the
%trained model
param = ml_initparam( param, ...
    struct( 'synthesize_images', false ) );

% optional parameter that if set to true, will process the images for
%visualization
param = ml_initparam( param, ...
    struct( 'postprocess_images', false ) );

% optional parameter that sets the number of synthesized images
param = ml_initparam( param, ...
    struct( 'numberOfSynthesizedImages', 100 ) );

param.model = model;
clear model

%list of available patterns
patterns = { 'Nuc', 'Mit', 'LAM', 'TfR' };
% patterns = {'TfR'};

%icaoberg 11/16/2012
url = 'http://murphylab.web.cmu.edu/data/Hela/3D/multitiff';

if param.faster
    disp( 'Running demo in fast mode' );
    imageDirectory = [ pwd filesep 'images' filesep 'processed' ];
else
    imageDirectory = [ pwd filesep 'images' filesep 'raw' ];
end

%% Model training
if param.train_model
    if exist( imageDirectory, 'dir' ) && length( dir(imageDirectory) ) == 196
        disp('Image directory found and built');
    else
        compressed_file = 'cellorganizer_images.zip';
        if ~exist( compressed_file, 'file' )
            disp([ 'Downloading Dataset: ' compressed_file ]);
            urlwrite([url '/' compressed_file], compressed_file);
            
            disp( 'Extracting files' );
            unzip( compressed_file );
        else
            disp( 'Pattern found on disk. Extracting files from existing tarball' );
            if ~exist( [pwd filesep compressed_file], 'file' )
                disp( 'Extracting files' );
                untar( compressed_file );
            else
                disp( 'Extracted folder found. Skipping extraction' );
            end
        end
    end
    
    dimensionality = '3D';
    pattern = 'framework';
    
    dna = [imageDirectory filesep '*cell*ch0*.tif'];
    cell = [imageDirectory filesep '*cell*ch1*.tif'];
    param.masks = [imageDirectory filesep '*mask*.tif'];
    
    % generic model options
    % ---------------------
    % model.name                (optional) Holds the name of the model. Default is empty.
    param.model.name = '3d_hela_framework_model';
    
    % model.id                  (optional) Holds the id of the model. Default is empty.
    try
        [status,result] = system( 'uuidgen' );
        param.model.id = result(1:end-1);
    catch err
        param.model.id = num2str(now);
    end
    
    % model.filename            Holds the output filename.
    param.model.filename = [ lower(pattern) '.xml' ];
    
    % nuclear shape model options
    % ---------------------------
    % nucleus.type              Holds the nuclear model type. Default is "cylindrical_surface".
    param.nucleus.type = 'cylindrical_surface';
    
    % nucleus.name              (optional) Holds the name of the nuclear model. Default is empty.
    param.nucleus.name = 'all';
    
    % nucleus.id                (optional) Holds the id of the nuclear model. Default is empty.
    try
        [status,result] = system( 'uuidgen' );
        param.nucleus.id = result(1:end-1);
    catch err
        param.nucleus.id = num2str(now);
    end
    
    % cell shape model options
    % ------------------------
    % cell.type                 Holds the cell model type. Default is "ratio".
    param.cell.type = 'ratio';
    
    % cell.name                 (optional) Holds the name of the cell model. Default is empty.
    param.cell.model = 'framework';
    
    % cell.id                   (optional) Holds the id the cell model. Default is empty.
    try
        [status,result] = system( 'uuidgen' );
        param.cell.id = result(1:end-1);
    catch err
        param.cell.id = num2str(now);
    end
    
    % other options
    % -------------
    % verbose                   (optional) Displays messages to screen. The default is true.
    param.verbose = true;
    
    % debug                     (optional) Reports errors and warnings. Default is false.
    param.debug = true;
    
    % train.flag                (optional) Selects what model is going to be trained ('nuclear',
    %                           'framework', or 'all'). Default is 'all'
    param.train.flag = 'framework';
    
    %this is the resolution of the datasets
    param.model.resolution = [0.049, 0.049, 0.2000];
    
    %documentation
    param.documentation.author = 'Murphy Lab';
    param.documentation.email = 'murphy@cmu.edu';
    param.documentation.website = 'murphy@cmu.edu';
    param.documentation.description = 'This is the framework trained on all images from the HeLa dataset.';
    
    answer = img2slml( dimensionality, dna, cell, [], param );
end

%% Image synthesis
if param.synthesize_images
    %try to synthesize images from model
    try
        state = rng( param.seed );
    catch
        state = RandStream.create('mt19937ar','seed', param.seed );
        RandStream.setDefaultStream(state);
    end
    
    outputDirectory = pwd;
    param.targetDirectory = [ outputDirectory ];
    param.prefix = 'synthesizedImages';
    param.compression = 'lzw';
    param.microscope = 'none';
    param.synthesis = 'framework';
    param.verbose = true;
    param.debug = true;
    
    slml2img( {'./framework.mat'}, param );
end

%% Image post-processing
if param.postprocess_images
    % raw images
    disp( 'Processing raw images' );
    input_images_directory = [ pwd filesep 'images' filesep 'raw' ];
	output_images_directory = [ pwd filesep 'raw' ];
	answer = ...
	    raw_images_gallery_wrapper( ...
	    	input_images_directory, output_images_directory );
 
    % intermediate results
    disp( 'Processing intermediate results' )
	input_images_directory = [ pwd filesep 'intermediate_results' ];
	output_images_directory = [ pwd filesep 'intermediate_results_images' ];
	answer = ...
	    intermediate_results_image_gallery_wrapper( ...
	    	input_images_directory, output_images_directory );
 
    % preprocessed images
    disp( 'Processing preprocessing images' )
	input_images_directory = [ pwd filesep 'temp' filesep 'preprocessing' ];
	output_images_directory =  [ pwd filesep filesep 'preprocesed_images' ];
	answer = ...
	    preprocessed_images_gallery_wrapper( ...
	    	input_images_directory, output_images_directory );
 
    % syntehtic images
    disp( 'Processing synthetic images' )
	input_images_directory = [ pwd filesep 'synthesizedImages' ];
	output_images_directory = [ pwd filesep 'processed_synthetic_images' ];
	synthesized_images_gallery_wrapper( ...
		input_images_directory, output_images_directory );
 
    % image parameterization
    disp( 'Processing cell codes' )
	input_images_directory = [ pwd filesep 'temp' filesep 'cell_shape_eigen' ];
	output_images_directory = [ pwd filesep filesep 'cellcode_images' ];
	cellcodes_images_gallery_wrapper( ...
		input_images_directory, output_images_directory );
end